@@ -6,7 +6,7 @@ use warnings;
 use Perl::Critic::Utils qw(:severities :classification :ppi);
 use parent 'Perl::Critic::Policy';
 
-our $VERSION = '0.024';
+our $VERSION = '0.025';
 
 use constant DESC => 'Using $a or $b outside sort()';
 use constant EXPL => '$a and $b are special package variables for use in sort() and related functions. Declaring them as lexicals like "my $a" may break sort(). Use different variable names.';
@@ -25,44 +25,32 @@ sub default_severity { $SEVERITY_HIGH }
 sub default_themes { 'freenode' }
 sub applies_to { 'PPI::Token::Symbol' }
 
-my %sorters = (
-	sort      => 1,
-	reduce    => 1,
-	pairgrep  => 1,
-	pairfirst => 1,
-	pairmap   => 1,
-	pairwise  => 1,
-);
+my @sorters = qw(sort reduce pairgrep pairfirst pairmap pairwise);
 
 sub violates {
 	my ($self, $elem) = @_;
 	return () unless $elem->symbol eq '$a' or $elem->symbol eq '$b';
+
+	my %sorters_hash = map { ($_ => 1) } @sorters, keys %{$self->{_extra_pair_functions}};
+	my $found = $self->_find_sorter($elem, \%sorters_hash);
 	
-	$sorters{$_} = 1 foreach keys %{$self->{_extra_pair_functions}};
-	
-	my $name = $self->_find_sorter($elem);
-	return $self->violation(DESC, EXPL, $elem) unless exists $sorters{$name};
+	return $self->violation(DESC, EXPL, $elem) unless $found;
 	return ();
 }
 
 sub _find_sorter {
-	my ($self, $elem) = @_;
+	my ($self, $elem, $sorters) = @_;
 	
-	my $outer = $elem;
+	my $outer = $elem->parent;
 	$outer = $outer->parent until !$outer or $outer->isa('PPI::Structure::Block');
-	return '' unless $outer and $outer->isa('PPI::Structure::Block');
+	return '' unless $outer;
 	
 	# Find function or method call (assumes block/sub is first argument)
 	my $function = $outer->previous_token;
 	$function = $function->previous_token until !$function
-		or ($function->isa('PPI::Token::Word')
-			and (is_method_call $function or is_function_call $function or is_hash_key $function));
-	return $self->_find_sorter($outer) unless $function and $function->isa('PPI::Token::Word')
-		and (is_method_call $function or is_function_call $function or is_hash_key $function);
-	
-	my $name = $function;
-	$name =~ s/.+:://;
-	return $name;
+		or ($function->isa('PPI::Token::Word') and $function =~ m/([^:]+)\z/ and exists $sorters->{$1});
+	return $self->_find_sorter($outer) unless $function;
+	return $function;
 }
 
 1;
